@@ -10,5 +10,5 @@ Why does this file exist, and why __main__? For more info, read:
 """
 from redisboard.cli import main
 
-if __name__ == "__main__":
+if __name__ == '__main__':
     main()
