@@ -1,17 +1,25 @@
 from functools import partial
 from functools import wraps
+from logging import getLogger
+from typing import Union
+from urllib.parse import quote
+from urllib.parse import unquote_to_bytes
 
 from django.contrib import admin
 from django.http import HttpResponseForbidden
+from django.http import HttpResponseNotFound
 from django.shortcuts import get_object_or_404
+from django.shortcuts import render
 from django.shortcuts import resolve_url
 from django.template.response import TemplateResponse
 from django.utils.html import format_html
 from django.utils.translation import gettext_lazy as _
 
+from .data import REDISBOARD_SCAN_COUNT
 from .models import RedisServer
-from .views import inspect
-from .views import inspect_key
+from .structs import DBInfo
+
+logger = getLogger(__name__)
 
 
 def cleanup_changelist_response(response: TemplateResponse):
@@ -120,35 +128,99 @@ class RedisServerAdmin(admin.ModelAdmin):
         return [
             path(
                 '<int:server_id>/inspect/',
-                wrap(inspect),
+                wrap(self.inspect),
                 name='redisboard_redisserver_inspect',
             ),
             path(
                 '<int:server_id>/inspect/<int:db>/',
-                wrap(inspect),
+                wrap(self.inspect),
                 name='redisboard_redisserver_inspect',
             ),
             path(
                 '<int:server_id>/inspect/<int:db>/key/<path:key>/',
-                wrap(inspect_key),
+                wrap(self.inspect_key),
                 name='redisboard_redisserver_inspect',
             ),
             path(
                 '<int:server_id>/inspect/<int:db>/<int:cursor>/',
-                wrap(inspect),
+                wrap(self.inspect),
                 name='redisboard_redisserver_inspect',
             ),
             path(
                 '<int:server_id>/inspect/<int:db>/<int:cursor>/key/<path:key>/',
-                wrap(inspect_key),
+                wrap(self.inspect_key),
                 name='redisboard_redisserver_inspect',
             ),
             path(
                 '<int:server_id>/inspect/<int:db>/<int:cursor>/<int:count>/key/<path:key>/',
-                wrap(inspect_key),
+                wrap(self.inspect_key),
                 name='redisboard_redisserver_inspect',
             ),
         ] + urlpatterns
 
+    def inspect_key(self, request, server: RedisServer, db: int, key: str, cursor: int = 0, count: int = 0):
+        key: bytes = unquote_to_bytes(key)
+        display = server.display
+        if not server.connection.exists(key):
+            return HttpResponseNotFound('Key is gone.')
+        stats = display.keys(db, [key])
+        scan = display.value(db, key, cursor=cursor, count=count)
+        return render(
+            request,
+            'redisboard/inspect_key.html',
+            {
+                **self.admin_site.each_context(request),
+                'key': display.decoder.key(key),
+                'encoded_key': quote(key),
+                'stats': stats,
+                'count': scan.count + count,
+                'scan': scan,
+                'db': {
+                    'id': db,
+                    'cursor': cursor,
+                },
+                'original': server,
+                'opts': {'app_label': 'redisboard', 'object_name': 'redisserver'},
+                'media': '',
+            },
+        )
+
+    def inspect(self, request, server: RedisServer, db: Union[int, None] = None, cursor: Union[int, None] = 0):
+        stats = server.stats
+        active = None
+        databases = []
+        if stats:
+            total_keys = sum(details.get('keys', 0) for details in stats.databases.values())
+            if db is not None:
+                if db in stats.databases:
+                    active = DBInfo(db, stats.databases[db], scan=True)
+                else:
+                    active = DBInfo(db, {'keys': 0}, scan=True)
+                databases = [active]
+            elif total_keys < REDISBOARD_SCAN_COUNT:
+                databases = [DBInfo(*item, scan=True) for item in stats.databases.items()]
+            else:
+                databases = [DBInfo(*item) for item in stats.databases.items()]
+
+            for dbinfo in databases:
+                if dbinfo.scan:
+                    dbinfo.scan = server.display.scan(dbinfo.id, cursor=cursor, **request.GET.dict())
+                    dbinfo.cursor = cursor
+        return render(
+            request,
+            'redisboard/inspect.html',
+            {
+                **self.admin_site.each_context(request),
+                'databases': databases,
+                'display': server.display,
+                'original': server,
+                'stats': stats,
+                'active': active,
+                'filters': f'?{request.GET.urlencode()}' if request.GET else '',
+                'opts': {'app_label': 'redisboard', 'object_name': 'redisserver'},
+                'media': '',
+            },
+        )
+
 
 admin.site.register(RedisServer, RedisServerAdmin)
